#!/bin/bash
#
# Raspberry Pi 2/3 and  Wireless setup using systemd networking. This is not 
# compatible with netctld
# You must have your system configured using DHCP and not static IP addressing. 
# For this release, static IP addressing does not play nicely together.

# $Id: wireless-setup.sh 7 2016-07-01 18:40:42Z w0anm $

# ---------------
# Copyright (C) 2016 Christopher Kovacs, W0ANM
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
# ---------------
#
#  Modified for no passphrase (open) access and SSID's with spaces
#  D. Crompton, WA3DSP - 6/29/2016

# tmp scan file
WL_SCAN_FILE=/tmp/wireless_scan.info

WL_DEV=wlan0

# wpa supplicant file for wlan0
WPA_SUPP_CONF=/etc/wpa_supplicant/wpa_supplicant_custom-wlan0.conf
#WPA_SUPP_CONF=/root/test.conf
# functions

ssid_view () {

    iwlist wlan0 scan > $WL_SCAN_FILE
 
    if (grep 'No scan results' $WL_SCAN_FILE > /dev/null) ; then
        dialog --msgbox "No scan results." 5 20
        exit 1
    fi 
        
    entries=$(cat $WL_SCAN_FILE)

    SAVEIFS=$IFS
    IFS=$(echo -en "\n\b")
    count=0
    for line in $entries
       do
          entry=$line
 
#          essid=$(echo $entry | grep ESSID: | awk '{print $1}')
# Added whitespace removal but not within SSID - WA3DSP 6/29/2016
	  essid=$(echo $entry | grep ESSID: | sed 's/^[ \t]*//;s/[ \t]*$//')   #  sed -e 's/^[ \t]*//')
          essid=${essid#ESSID:}

          # stip off any quotes
          essid=$(echo ${essid} | sed -e 's/^"//' -e 's/"$//')

          if [ ! "$essid" = "" ] ; then
              ESSID[$count]=$essid
          fi 
          quality=$(echo $entry | grep Quality | awk '{print $1}' | awk 'BEGIN {FS="="} { print $2 } ')
          ##quality=$(echo $entry | grep Quality | awk '{print $1}')
          if [ ! "$quality" = "" ] ; then
              QUALITY[$count]=$quality
          fi
          signal=$(echo $entry | grep Quality | awk '{print $1}' | awk 'BEGIN {FS="="} { print $2 } ')
          #signal=$(echo $entry | grep Quality | awk '{ print $3 } ')
          if [ ! "$signal" = "" ] ; then
              SIGNAL[$count]=$signal
              count=$(expr $count + 1) 
          fi
      done

  for ptr in $(seq 1 $(expr $count - 1)) ; do
      if (grep "${ESSID[$ptr]}" /etc/wpa_supplicant/wpa_supplicant_custom-wlan0.conf> /dev/null) ; then
        ENTRY[$ptr]="**"
        printf "   %s\t **%-20s\t %-8s\t %s\n" $ptr ${ESSID[$ptr]} ${QUALITY[$ptr]} ${SIGNAL[$ptr]}
      else
        ENTRY[$ptr]="  "
        printf "   %s\t  %-20s\t %-8s\t %s\n" $ptr ${ESSID[$ptr]} ${QUALITY[$ptr]} ${SIGNAL[$ptr]}
      fi
  done

  SSID_SEL=$(dialog --radiolist "Below shows SSID Signal Quality and if entry present. Select SSID:" 20 60 10 \
    "1"  "${ESSID[1]} -  ${QUALITY[1]} ${ENTRY[1]} " on  \
    "2"  "${ESSID[2]} - ${QUALITY[2]} ${ENTRY[2]} "  off \
    "3"  "${ESSID[3]} - ${QUALITY[3]} ${ENTRY[3]} "  off \
    "4"  "${ESSID[4]} - ${QUALITY[4]} ${ENTRY[4]} "  off \
    "5"  "${ESSID[5]} - ${QUALITY[5]} ${ENTRY[5]} "  off \
    "6"  "${ESSID[6]} - ${QUALITY[6]} ${ENTRY[6]} "  off \
    "7"  "${ESSID[7]} - ${QUALITY[7]} ${ENTRY[7]} "  off \
    "8"  "${ESSID[8]} - ${QUALITY[8]} ${ENTRY[8]} "  off \
    "9"  "${ESSID[9]} - ${QUALITY[9]} ${ENTRY[9]} "  off \
    "10"  "${ESSID[10]} - ${QUALITY[10]} ${ENTRY[10]} "  off \
    "11"  "${ESSID[11]} - ${QUALITY[11]} ${ENTRY[11]} "  off \
    "12"  "${ESSID[12]} - ${QUALITY[12]} ${ENTRY[12]} "  off \
    "13"  "${ESSID[13]} - ${QUALITY[13]} ${ENTRY[13]} "  off \
    "14"  "${ESSID[14]} - ${QUALITY[14]} ${ENTRY[14]} "  off \
    "15"  "${ESSID[15]} - ${QUALITY[15]} ${ENTRY[15]} "  off \
    "16"  "${ESSID[16]} - ${QUALITY[16]} ${ENTRY[16]} "  off \
    "17"  "${ESSID[17]} - ${QUALITY[17]} ${ENTRY[17]} "  off \
    "18"  "${ESSID[18]} - ${QUALITY[18]} ${ENTRY[18]} "  off \
    "19"  "${ESSID[19]} - ${QUALITY[19]} ${ENTRY[19]} "  off \
    "20"  "${ESSID[20]} - ${QUALITY[20]} ${ENTRY[20]} "  off 3>&1 1>&2 2>&3)

  if [ "$?" = 1 ] ; then
      exit 1
  fi

  ESSID_SEL=${ESSID[${SSID_SEL}]}

  IFS=$SAVEIFS
}  # end of function

### main

dialog --yesno "This program sets the SSID and Password for the wireless network.  Do you wish to continue?" 10 60
if [ "$?" = 1 ] ; then
    exit
fi
  
# if not file, create one.
if [ ! -f "$WPA_SUPP_CONF" ] ; then
  touch "$WPA_SUPP_CONF"
  REBOOTNEEDED=yes
fi

# bring up link
ip link set wlan0 up
  
dialog --infobox "Scanning WiFi SSID's, please wait.." 8 70

# scan network
ssid_view

# cjk
# check if already already configured, if so, prompt to remove and re-add 
# get the current entries
CONFIG_SSID=$(grep "ssid=" $WPA_SUPP_CONF | grep -v grep)

if (echo $CONFIG_SSID | grep "ssid=\"${ESSID_SEL}\"") ; then
    dialog --yes-button "Add" --no-button "Quit" --yesno  "${ESSID_SEL} SSID entry is already present in $WPA_SUPP_CONF file. Do you want to remove and re-add this entry or Quit?" 10 60 
    if [ "$?" = 0 ] ; then
        # remove entry
        foo="$(cat "$WPA_SUPP_CONF" | awk '/'$ESSID_SEL'/ { flag=1 }; flag==0 { print $0 }; /network={/ { flag=0 }' )"

        if echo -e "$foo" | tail -1 | grep -q 'network={'; then
            foo=$(echo -e "$foo" | head -n -1)
        fi
        echo "foo --> $foo"
        echo -e "$foo" > "$WPA_SUPP_CONF"
    else
        exit 1
    fi
fi

SSID=${ESSID_SEL}

PASSOK=false
while [ "$PASSOK" = false ] ; do
    WIREPASS=$(dialog --title "SSID Password" \
    --clear \
    --insecure \
    --inputbox "Enter your wireless network password (8-32 characters), enter blank for open network and press Ok to continue." 10 60 3>&1 1>&2 2>&3)

    if [ "$?" = 1 ] ; then
        exit
    fi

    # validate password, if not valid, re-ask passwd.
    # check entered length. must be 8 to 32 characters or none if open.
    # Made changes for no passphrase (open) access

    site="closed"

   if [ "${#WIREPASS}" -eq "0" ] ; then  
	dialog --msgbox "No passowrd entered assuming open site." 10 60
	site="open"
	PASSOK=true
    else 
    	if [ ${#WIREPASS} -lt 8 ] || [ ${#WIREPASS} -gt 32 ] ; then
        	dialog --msgbox "Pasword length must be between 8 and 32 characters in length." 10 60
        	PASSOK=false
    	else
        	PASSOK=true
    	fi
    fi
done



# if adding new SSID, append the the config file.
if [ "$site" == "open" ] ; then
	dialog --yesno "Adding \"open\" no passphrase site with SSID = \"$SSID\" - OK to Add?" 10 80
	if [ "$?" != "0"0 ] ; then
	    exit
	fi
	echo >> $WPA_SUPP_CONF
	echo "network={" >> $WPA_SUPP_CONF 
	echo -e "\tssid=\"$SSID\"" >> $WPA_SUPP_CONF
	echo -e "\tkey_mgmt=NONE" >> $WPA_SUPP_CONF
	echo "}" >> $WPA_SUPP_CONF
else
	dialog --yesno "Adding SSID = \"$SSID\" with passphrase = \"$WIREPASS\" - OK to Add?" 10 80
	if [ "$?" != "0" ] ; then
	    exit
	fi
	wpa_passphrase "$SSID" "$WIREPASS"  >> $WPA_SUPP_CONF
fi

# Restart the wireless network for next reboot
systemctl restart wpa_supplicant@wlan0.service
# you may be able to start the service or re-enable.

if [ "$REBOOTNEEDED" = "yes" ] ; then
    if (dialog --yesno "You will need to reboot the server, first remove the eth0 cable now if installed. Do you want to reboot the sever now?" 10 60) then
        shutdown -r now
    else
        dialog --msgbox "When you are ready to shutdown the server, the command is 'shutdown -r now'. " 10 60
    fi
fi


